@@ -31,7 +31,7 @@ __catfish_data_directory__ = '/usr/share/catfish/'
 # Location of locate.db file
 __locate_db_path__ = '/var/lib/mlocate/mlocate.db'
 __license__ = 'GPL-2+'
-__version__ = '1.4.13'
+__version__ = '4.16.0'
 __url__ = 'https://docs.xfce.org/apps/catfish/start'
 
 
@@ -86,7 +86,7 @@ def get_about():
     return {
         'version': __version__,
         'program_name': _('Catfish File Search'),
-        'icon_name': 'catfish',
+        'icon_name': 'org.xfce.catfish',
         'website': __url__,
         'comments': _('Catfish is a versatile file searching tool.'),
         'copyright': 'Copyright (C) 2007-2012 Christian Dywan <christian@twotoasts.de>\n'
